from enum import Enum, auto
from acceldata_sdk.models.asset import Asset
from acceldata_sdk.models.datasource import DataSource
from typing import List


class ExecutionPeriod(Enum):
    Last15minutes = auto()
    Last30minutes = auto()
    Last1hour = auto()
    Last3hours = auto()
    Last6hours = auto()
    Last12hours = auto()
    Last24hours = auto()
    Today = auto()
    Yesterday = auto()
    Last7days = auto()
    Thismonth = auto()
    Last1month = auto()
    Last3month = auto()


class RuleType(Enum):
    DATA_QUALITY = 'DATA-QUALITY'
    RECONCILIATION = 'RECONCILIATION'
    EQUALITY = 'EQUALITY'
    DATA_DRIFT = 'DATA-DRIFT'
    SCHEMA_DRIFT = 'SCHEMA_DRIFT'


class RuleExecutionMode(Enum):
    SCHEDULED = auto()
    MANUAL = auto()
    WEBHOOK = auto()
    API = auto()


class RuleExecutionStatus(Enum):
    STARTED = auto()
    RUNNING = auto()
    ERRORED = auto()
    WARNING = auto()
    SUCCESSFUL = auto()
    ABORTED = auto()


class RuleResultStatus(Enum):
    STARTED = auto()
    RUNNING = auto()
    ERRORED = auto()
    WARNING = auto()
    SUCCESSFUL = auto()
    ABORTED = auto()


class PolicyExecutionType(Enum):
    SAMPLE = auto()
    FULL = auto()
    INCREMENTAL = auto()


class RuleThresholdLevel:
    def __init__(self, success, warning=None, *args, **kwargs):
        self.success = success
        self.warning = warning

    def __repr__(self):
        return f"RuleThresholdLevel({self.__dict__})"


class LivyExecutorConfig:
    def __init__(self, executorMemory=None, executorCores=None, numExecutors=None, *args, **kwargs):
        self.executorMemory = executorMemory
        self.executorCores = executorCores
        self.numExecutors = numExecutors

    def __repr__(self):
        return f"LivyExecutorConfig({self.__dict__})"


class DataprocSparkResourceConfig:
    def __init__(self, numWorkerNodes=None, numMasterNodes=None, clusterWorkerType=None, clusterMasterType=None, *args,
                 **kwargs):
        self.numWorkerNodes = numWorkerNodes
        self.numMasterNodes = numMasterNodes
        self.clusterWorkerType = clusterWorkerType
        self.clusterMasterType = clusterMasterType

    def __repr__(self):
        return f"DataprocSparkResourceConfig({self.__dict__})"


class YunikornSparkResourceConfig:
    def __init__(self, executorMemory=None, executorCores=None, maxExecutors=None, minExecutors=None,
                 executorMemoryOverhead=None, driverMemory=None,
                 driverCores=None, driverMemoryOverhead=None):
        self.executorMemory = executorMemory
        self.executorCores = executorCores
        self.maxExecutors = maxExecutors
        self.minExecutors = minExecutors
        self.executorMemoryOverhead = executorMemoryOverhead
        self.driverMemory = driverMemory
        self.driverCores = driverCores
        self.driverMemoryOverhead = driverMemoryOverhead


class DataBricksExecutorConfig:
    def __init__(self, minWorkers=None, maxWorkers=None, clusterWorkerType=None, clusterDriverType=None, *args, **kwargs):
        self.minWorkers = minWorkers
        self.maxWorkers = maxWorkers
        self.clusterWorkerType = clusterWorkerType
        self.clusterDriverType = clusterDriverType

    def __repr__(self):
        return f"DataBricksExecutorConfig({self.__dict__})"


class ExecutorConfig:
    def __init__(self, livy=None, databricks=None, dataproc=None, yunikorn=None, additionalConfiguration=None, *args,
                 **kwargs):
        if isinstance(livy, dict):
            self.livy = LivyExecutorConfig(**livy)
        else:
            self.livy = livy
        if isinstance(databricks, dict):
            self.databricks = DataBricksExecutorConfig(**databricks)
        else:
            self.databricks = databricks
        if isinstance(dataproc, dict):
            self.dataproc = DataprocSparkResourceConfig(**databricks)
        else:
            self.dataproc = dataproc
        if isinstance(yunikorn, dict):
            self.yunikorn = YunikornSparkResourceConfig(**databricks)
        else:
            self.yunikorn = yunikorn
        self.additionalConfiguration = additionalConfiguration


    def __repr__(self):
        return f"ExecutorConfig({self.__dict__})"

class SparkResourceConfig:
    def __init__(self, livy=None, databricks=None, dataproc=None, yunikorn=None, additionalConfiguration=None, *args,
                 **kwargs):
        if isinstance(livy, dict):
            self.livy = LivyExecutorConfig(**livy)
        else:
            self.livy = livy
        if isinstance(databricks, dict):
            self.databricks = DataBricksExecutorConfig(**databricks)
        else:
            self.databricks = databricks
        if isinstance(dataproc, dict):
            self.dataproc = DataprocSparkResourceConfig(**databricks)
        else:
            self.dataproc = dataproc
        if isinstance(yunikorn, dict):
            self.yunikorn = YunikornSparkResourceConfig(**databricks)
        else:
            self.yunikorn = yunikorn
        self.additionalConfiguration = additionalConfiguration


    def __repr__(self):
        return f"ExecutorConfig({self.__dict__})"

class RuleExecutionSummary:
    def __init__(self, ruleId=None, executionMode=None, executionStatus=None, resultStatus=None, startedAt=None,
                 executionType=None, thresholdLevel=None, ruleVersion=None, id=None, ruleName=None, ruleType=None,
                 lastMarker=None, leftLastMarker=None, rightLastMarker=None, executionError=None, finishedAt=None,
                 resetPoint=None, persistencePath=None, resultPersistencePath=None, executorConfig=None,
                 markerConfig=None, leftMarkerConfig=None, rightMarkerConfig=None, dataPersistenceEnabled=None,
                 isProtectedResource=None, sparkResourceConfig=None, *args, **kwargs):

        self.ruleId = ruleId
        self.id = id
        self.ruleName = ruleName
        if isinstance(ruleType, dict):
            self.ruleType = RuleType(**ruleType)
        else:
            self.ruleType = ruleType
        if isinstance(executionMode, dict):
            self.executionMode = RuleExecutionMode(**executionMode)
        else:
            self.executionMode = executionMode
        if isinstance(executionStatus, dict):
            self.executionStatus = RuleExecutionStatus(**executionStatus)
        else:
            self.executionStatus = executionStatus
        if isinstance(resultStatus, dict):
            self.resultStatus = RuleResultStatus(**resultStatus)
        else:
            self.resultStatus = resultStatus

        self.lastMarker = lastMarker
        self.leftLastMarker = leftLastMarker
        self.rightLastMarker = rightLastMarker
        self.executionError = executionError
        self.startedAt = startedAt
        self.finishedAt = finishedAt

        if isinstance(thresholdLevel, dict):
            self.thresholdLevel = RuleThresholdLevel(**thresholdLevel)
        else:
            self.thresholdLevel = thresholdLevel
        self.resetPoint = resetPoint
        self.persistencePath = persistencePath
        self.resultPersistencePath = resultPersistencePath
        self.ruleVersion = ruleVersion
        if isinstance(executorConfig, dict):
            self.executorConfig = ExecutorConfig(**executorConfig)
        else:
            self.executorConfig = executorConfig

        if isinstance(sparkResourceConfig, dict):
            self.sparkResourceConfig = SparkResourceConfig(**executorConfig)
        else:
            self.sparkResourceConfig = sparkResourceConfig

        self.markerConfig = markerConfig
        self.leftMarkerConfig = leftMarkerConfig
        self.rightMarkerConfig = rightMarkerConfig
        if isinstance(executionType, dict):
            self.executionType = PolicyExecutionType(**executionType)
        else:
            self.executionType = executionType
        self.dataPersistenceEnabled = dataPersistenceEnabled
        self.isProtectedResource = isProtectedResource

    def __repr__(self):
        return f"RuleExecutionSummary({self.__dict__})"


class RuleExecutionResult:
    def __init__(self, status, description=None, successCount=None, failureCount=None, qualityScore=None, *args, **kwargs):
        if isinstance(status, dict):
            self.status = RuleResultStatus(**status)
        else:
            self.status = status
        self.description = description
        self.successCount = successCount
        self.failureCount = failureCount
        self.qualityScore = qualityScore


class RuleItemResult:
    def __init__(self, id, ruleItemId, threshold, weightage, isWarning, resultPercent=None, success=None, error=None,
                 *args, **kwargs):
        self.id = id
        self.ruleItemId = ruleItemId
        self.threshold = threshold
        self.resultPercent = resultPercent
        self.success = success
        self.error = error
        self.weightage = weightage
        self.isWarning = isWarning

    def __repr__(self):
        return f"RuleItemResult({self.__dict__})"


class RuleResult:
    def __init__(self, execution: RuleExecutionSummary = None, items: [RuleItemResult] = None, meta=None,
                 result: RuleExecutionResult = None, *args, **kwargs):
        self.execution = execution
        self.items = items
        self.meta = meta
        self.result = result

    def __repr__(self):
        return f"RuleResult({self.__dict__})"


class PolicyFilter:
    def __init__(self, period: ExecutionPeriod = None, tags: str=None, lastExecutionResult:  RuleResultStatus = None,
                 asset: List[Asset] = None, policyType: RuleType = None, enable: bool = None, active: bool = None,
                 data_source: List[DataSource] = None, *args, **kwargs):
        self.period = period
        self.tags = tags
        self.lastExecutionResult = lastExecutionResult
        self.assets = asset
        self.data_sources = data_source
        self.policyType = policyType
        self.enable = enable
        self.active = active

    def __repr__(self):
        return f"PolicyFilter({self.__dict__})"


import logging
import pytest
import test_constants as test_const
from test_commons import retry_operation
from acceldata_sdk.torch_client import TorchClient
from acceldata_sdk.models.profile import StartProfilingRequest
from acceldata_sdk.models.common_types import (
    ExecutionType, BoundsIdMarkerConfig, BoundsDateTimeMarkerConfig,
    BoundsFileEventMarkerConfig, TimestampBasedMarkerConfig
)
from acceldata_sdk.errors import APIError, TorchSdkException

# Logging configuration
LOGGER = logging.getLogger(__name__)
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    handlers=[logging.StreamHandler()]
)

# Initialize Torch client
torch_client = TorchClient(**test_const.torch_credentials)
LOGGER.info("Torch client connected")


@pytest.fixture(scope="module")
def asset():
    return torch_client.get_asset(identifier=test_const.table_asset_uid)


@pytest.mark.order(1)
def test_execute_full_profiling_backward_compatible(asset):
    LOGGER.info(f"Asset ID: {asset.id}")
    LOGGER.info("\n" + "=" * 20 + " Executing FULL profile using backward compatible call " + "=" * 20 + "\n")

    def operation():
        return asset.start_profile(profiling_type=ExecutionType.FULL)

    profiling_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    LOGGER.info(profiling_execution_result)
    assert profiling_execution_result is not None


@pytest.mark.order(2)
def test_execute_incremental_profiling_backward_compatible(asset):
    LOGGER.info(f"Asset ID: {asset.id}")
    LOGGER.info("\n" + "=" * 20 + " Executing INCREMENTAL profile using backward compatible call " + "=" * 20 + "\n")

    def operation():
        return asset.start_profile(profiling_type=ExecutionType.INCREMENTAL)

    profiling_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    LOGGER.info(profiling_execution_result)
    assert profiling_execution_result is not None


@pytest.mark.order(3)
def test_execute_full_profiling_using_start_profiling_request(asset):
    LOGGER.info(f"Asset ID: {asset.id}")
    profiling_execution_request = StartProfilingRequest(
        profilingType=ExecutionType.FULL
    )
    LOGGER.info(f" Executing FULL profile using Profiling Execution Request: {profiling_execution_request}")

    def operation():
        return asset.start_profile(start_profiling_request=profiling_execution_request)

    profiling_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    LOGGER.info(f"Profiling Execution Result: {profiling_execution_result}")
    assert profiling_execution_result is not None


@pytest.mark.order(4)
def test_execute_incremental_profiling_using_start_profiling_request(asset):
    LOGGER.info(f"Asset ID: {asset.id}")
    profiling_execution_request = StartProfilingRequest(profilingType=ExecutionType.INCREMENTAL)
    LOGGER.info(f" Executing INCREMENTAL profile using Profiling Execution Request: {profiling_execution_request}")

    def operation():
        return asset.start_profile(start_profiling_request=profiling_execution_request)

    profiling_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    LOGGER.info(f"Profiling Execution Result: {profiling_execution_result}")
    assert profiling_execution_result is not None


@pytest.mark.order(5)
def test_execute_selective_profiling_without_marker_configs(asset):
    LOGGER.info(f"Asset ID: {asset.id}")
    profiling_execution_request = StartProfilingRequest(profilingType=ExecutionType.SELECTIVE)
    LOGGER.info(f"Executing SELECTIVE profile without MARKER CONFIG: {profiling_execution_request}")
    try:
        profiling_execution_result = asset.start_profile(start_profiling_request=profiling_execution_request)
    except TorchSdkException as e:
        error_message = str(e)
        assert "markerConfig is a mandatory parameter for execution type" in error_message, \
            f"Unexpected error message: {error_message}"
        LOGGER.warning(f"Expected error occurred: {error_message}")


@pytest.mark.order(6)
def test_execute_selective_profiling_id_based(asset):
    LOGGER.info(f"Asset ID: {asset.id}")
    markerConfig = BoundsIdMarkerConfig(idColumnName="ID", fromId=0, toId=1000)
    profiling_execution_request = StartProfilingRequest(profilingType=ExecutionType.SELECTIVE,
                                                        markerConfig=markerConfig)
    LOGGER.info(
        f" Executing SELECTIVE ID based profiling  using Profiling Execution Request: {profiling_execution_request}")

    def operation():
        return asset.start_profile(start_profiling_request=profiling_execution_request)

    profiling_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    LOGGER.info(f"Profiling Execution Result: {profiling_execution_result}")
    LOGGER.info(profiling_execution_result)
    assert profiling_execution_result is not None


@pytest.mark.order(7)
def test_execute_selective_profiling_date_time_based(asset):
    LOGGER.info(f"Asset ID: {asset.id}")
    markerConfig = BoundsDateTimeMarkerConfig(dateColumnName="TO_DATE", format="yyyy-MM-dd",
                                              fromDate="2023-07-01 00:00:00.000", toDate="2024-07-14 23:59:59.999",
                                              timeZoneId="Asia/Calcutta")
    profiling_execution_request = StartProfilingRequest(profilingType=ExecutionType.SELECTIVE,
                                                        markerConfig=markerConfig)
    LOGGER.info(
        f" Executing SELECTIVE DATE TIME based profiling  using Profiling Execution Request: {profiling_execution_request}")

    def operation():
        return asset.start_profile(start_profiling_request=profiling_execution_request)

    profiling_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    LOGGER.info(f"Profiling Execution Result: {profiling_execution_result}")
    LOGGER.info(profiling_execution_result)
    assert profiling_execution_result is not None


@pytest.mark.order(8)
def test_execute_selective_profiling_file_event_based():
    asset = torch_client.get_asset(identifier=test_const.file_based_asset_uid)
    LOGGER.info(f"Asset ID: {asset.id}")
    markerConfig = BoundsFileEventMarkerConfig(
        fromDate="2019-04-01 00:00:00.000", toDate="2024-07-16 23:59:59.999",
        timeZoneId="Asia/Calcutta")
    profiling_execution_request = StartProfilingRequest(profilingType=ExecutionType.SELECTIVE,
                                                        markerConfig=markerConfig)
    LOGGER.info(
        f" Executing SELECTIVE FILE EVENT based profiling using Profiling Execution Request: {profiling_execution_request}")

    def operation():
        return asset.start_profile(start_profiling_request=profiling_execution_request)

    profiling_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    LOGGER.info(f"Profiling Execution Result: {profiling_execution_result}")
    LOGGER.info(profiling_execution_result)
    assert profiling_execution_result is not None
    LOGGER.info(profiling_execution_result)


# TODO Blocked on AOC-16709
@pytest.mark.order(9)
def test_execute_selective_profiling_kafka_timestamp():
    asset = torch_client.get_asset(identifier=test_const.kafka_asset_uid)
    LOGGER.info(f"Asset ID: {asset.id}")
    markerConfig = TimestampBasedMarkerConfig(
        format="yyyy-mm-dd",
        initialOffset="2023-06-01",
        timeZoneId="Asia/Calcutta")
    profiling_execution_request = StartProfilingRequest(profilingType=ExecutionType.SELECTIVE,
                                                        markerConfig=markerConfig)

    LOGGER.info(
        f" Executing SELECTIVE KAFKA TIMESTAMP based profiling using Profiling Execution Request: {profiling_execution_request}")

    def operation():
        return asset.start_profile(start_profiling_request=profiling_execution_request)

    profiling_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    LOGGER.info(f"Profiling Execution Result: {profiling_execution_result}")
    LOGGER.info(profiling_execution_result)
    assert profiling_execution_result is not None



from time import sleep
import logging
from acceldata_sdk.constants import RuleExecutionStatus, FailureStrategy, PolicyType
from acceldata_sdk.models.ruleExecutionResult import RuleResult, RuleExecutionSummary
from acceldata_sdk.models.common_types import PolicyExecutionRequest
from acceldata_sdk.errors import APIError, TorchSdkException

LOGGER = logging.getLogger("common")


def get_callables(rule_type, torch_client):
    LOGGER.info(f"Fetching the appropriate callable methods for the provided rule type: {rule_type.name}")
    if rule_type.value == PolicyType.RECONCILIATION.value:
        result_call = torch_client.get_reconciliation_rule_result
        exec_call = torch_client.execute_reconciliation_rule
    elif rule_type.value == PolicyType.DATA_QUALITY.value:
        result_call = torch_client.get_dq_rule_result
        exec_call = torch_client.execute_dq_rule
    else:
        raise TorchSdkException(f'{rule_type} not found. Please provide correct rule_type.'
                                f' Allowed values are {PolicyType.RECONCILIATION.value}, {PolicyType.DATA_QUALITY.value}')
    return exec_call, result_call

class Executor:
    def __init__(self, rule_type: PolicyType, torch_client, sync=False):
        self.id = None
        self.errorMessage = None
        self.sync = sync
        self.client = torch_client
        self.exec_call, self.result_call = get_callables(rule_type, torch_client)

    def is_id_valid(self):
        if self.id is None:
            execution_details = RuleResult()
            execution_details.execution = RuleExecutionSummary(
                executionStatus=RuleExecutionStatus.ERRORED, resultStatus=RuleExecutionStatus.ERRORED,
                executionError='valid execution_id is required.')
            LOGGER.info(execution_details.execution.executionError)
            return execution_details
        else:
            return True

    def get_status(self) -> RuleExecutionStatus:
        execution_id_check_passed = self.is_id_valid()
        if execution_id_check_passed is not True:
            return RuleExecutionStatus.ERRORED
        else:
            return self.get_execution_status(self.id)

    def get_execution_status(self, execution_id) -> RuleExecutionStatus:
        LOGGER.info('Getting rule execution status.')
        try:
            execution_details = self.result_call(execution_id=execution_id)
            return_value = RuleExecutionStatus[execution_details.execution.resultStatus]
        except APIError as e:
            LOGGER.info(f'get_status failed due to http error. Details:{e}')
            return_value = RuleExecutionStatus.ERRORED
        except Exception as e:
            return_value = RuleExecutionStatus.ERRORED
            LOGGER.info(f'get_status failed due to Exception. Details:{e}')
            LOGGER.info(self.execution_details.execution.executionError)
        return return_value

    def get_result(self, sleep_interval=5, total_retries=0, failure_strategy: FailureStrategy = FailureStrategy.DoNotFail) -> RuleResult:
        execution_id_check_passed = self.is_id_valid()
        if execution_id_check_passed is not True:
            if failure_strategy >= FailureStrategy.FailOnError:
                raise TorchSdkException(execution_id_check_passed.execution.executionError)
            else:
                return execution_id_check_passed
        else:
            return self.get_execution_result(self.id, sleep_interval, total_retries, failure_strategy)

    def get_execution_result(self, execution_id, sleep_interval=5, total_retries=0,
                             failure_strategy: FailureStrategy = FailureStrategy.DoNotFail) -> RuleResult:
        retry_count = 0
        try:
            while 1:
                execution_details = self.result_call(execution_id=execution_id)
                LOGGER.info(f'Checking rule execution result')
                if RuleExecutionStatus[execution_details.execution.executionStatus] == RuleExecutionStatus.RUNNING:
                    sleep(sleep_interval)
                    retry_count = retry_count + 1
                    if (total_retries == 0) or (total_retries > 0 and retry_count < total_retries):
                        continue
                    else:
                        execution_details.execution.executionError = f'Exiting after {total_retries} retries.'
                        LOGGER.info(execution_details.execution.executionError)
                        break
                else:
                    LOGGER.info('Rule completed.')
                    break
        except APIError as e:
            execution_details = RuleResult()
            execution_details.execution = RuleExecutionSummary(
                executionStatus=RuleExecutionStatus.ERRORED, resultStatus=RuleExecutionStatus.ERRORED,
                executionError=f'Rule execution failed due to http error. Details:{e}')
            LOGGER.info(execution_details.execution.executionError)
            if failure_strategy >= FailureStrategy.FailOnError:
                raise TorchSdkException(execution_details.execution.executionError)
        except Exception as e:
            execution_details = RuleResult()
            execution_details.execution = RuleExecutionSummary(
                executionStatus=RuleExecutionStatus.ERRORED, resultStatus=RuleExecutionStatus.ERRORED,
                executionError=f'Rule execution failed due to Exception. Details:{e}')
            LOGGER.info(execution_details.execution.executionError)
            if failure_strategy >= FailureStrategy.FailOnError:
                raise TorchSdkException(execution_details.execution.executionError)

        if RuleExecutionStatus[execution_details.execution.resultStatus] == RuleExecutionStatus.SUCCESSFUL:
            LOGGER.info('Rule completed successfully.')
        elif RuleExecutionStatus[execution_details.execution.resultStatus] == RuleExecutionStatus.WARNING:
            LOGGER.info('Rule completed with warnings.')
            if failure_strategy == FailureStrategy.FailOnWarning:
                raise TorchSdkException(f'Execution completed with warning. Details: {execution_details}')
        else:
            LOGGER.info(f'Rule execution completed with errors. Details:{execution_details}')
            if failure_strategy >= FailureStrategy.FailOnError:
                raise TorchSdkException(f'Rule execution completed with errors. Details: {execution_details}')
        return execution_details

    def execute(self, rule_id, incremental=False, sleep_interval=5, total_retries=0,
                failure_strategy: FailureStrategy = FailureStrategy.DoNotFail, pipeline_run_id=None,
                policy_execution_request: PolicyExecutionRequest = None):
        try:
            LOGGER.info(f"Policy Execution Request: {policy_execution_request.to_dict() if policy_execution_request else 'None'}")
            execution_obj = self.exec_call(rule_id=rule_id, incremental=incremental, pipeline_run_id=pipeline_run_id,
                                           policy_execution_request=policy_execution_request)
            self.id = execution_obj.id
            if self.sync:
                self.get_execution_result(self.id, sleep_interval, total_retries, failure_strategy)
        except APIError as e:
            self.errorMessage = f'Rule execution failed due to http error. Details:{e}'
            LOGGER.info(self.errorMessage)
            if failure_strategy >= FailureStrategy.FailOnError:
                raise TorchSdkException(self.errorMessage)
        except Exception as e:
            self.errorMessage = f'Rule execution failed due to Exception. Details:{e}'
            LOGGER.error(self.errorMessage)
            if failure_strategy >= FailureStrategy.FailOnError:
                raise TorchSdkException(self.errorMessage)

    def cancel(self):
        try:
            self.client.cancel_rule_execution(self.id)
        except APIError as e:
            self.errorMessage = f'Rule cancellation failed due to http error. Details:{e}'
            LOGGER.info(self.errorMessage)

        except Exception as e:
            self.errorMessage = f'Rule cancellation failed due to Exception. Details:{e}'
            LOGGER.info(self.errorMessage)





from acceldata_sdk.torch_client import TorchClient
import test_constants as test_const
import pprint
from acceldata_sdk.models.tags import AssetLabel, CustomAssetMetadata
from acceldata_sdk.models.profile import Profile, ProfileRequest, ProfilingType
from acceldata_sdk.errors import APIError, TorchSdkException
import time
from test_commons import retry_operation

pp = pprint.PrettyPrinter(indent=4)


class TestDS:
    torch_client = TorchClient(**test_const.torch_credentials)

    def test_get_datasource(self):
        ds_name = self.torch_client.get_datasource(test_const.ds_name, True)
        assert ds_name is not None

    def test_get_datasource_id(self):
        ds_id = self.torch_client.get_datasource(test_const.ds_id, False)
        assert ds_id is not None

    def test_get_all_datsource(self):
        dss = self.torch_client.get_datasources()
        assert dss is not None

    def test_get_ds_crawler(self):
        ds_id = self.torch_client.get_datasource(test_const.ds_id, True)
        status = ds_id.get_crawler_status()
        assert status is not None

    def test_start_crawler(self):
        ds_name = self.torch_client.get_datasource(test_const.ds_name, False)

        def operation():
            return ds_name.start_crawler()

        start_crawler = retry_operation(
            operation,
            test_const.MAX_RETRIES,
            test_const.RETRY_INTERVAL)
        time.sleep(15)
        status = ds_name.get_crawler_status()
        assert status is not None


class TestAsset:
    torch_client = TorchClient(**test_const.torch_credentials)

    def test_get_asset(self):
        asset = self.torch_client.get_asset(test_const.asset_uid)
        assert asset is not None

    def test_get_asset_id(self):
        asset = self.torch_client.get_asset(test_const.asset_id)
        assert asset is not None

    def test_get_asset_metadata(self):
        asset = self.torch_client.get_asset(test_const.asset_id)
        metadata_asset = asset.get_metadata()
        assert metadata_asset is not None

    def test_get_asset_sample_data(self):
        asset = self.torch_client.get_asset(test_const.asset_id)
        sample_data_asset = asset.sample_data()
        assert sample_data_asset is not None

    def test_get_asset_labels(self):
        asset = self.torch_client.get_asset(test_const.asset_id)
        labels_asset = asset.get_labels()
        assert labels_asset is not None

    def test_add_asset_labels(self):
        asset = self.torch_client.get_asset(test_const.asset_id)
        asset.add_labels(labels=[AssetLabel('test12', 'shubh12'), AssetLabel('test22', 'shubh32')])
        labels_asset = asset.get_labels()
        labels_asset = asset.get_labels()
        assert labels_asset is not None

    def test_add_asset_custom_metadata(self):
        asset = self.torch_client.get_asset(test_const.asset_id)
        asset.add_custom_metadata(
            custom_metadata=[CustomAssetMetadata('testcm1', 'shubhcm1'), CustomAssetMetadata('testcm2', 'shubhcm2')])
        metadata_asset = asset.get_metadata()
        assert metadata_asset is not None

    def test_profile_status(self):
        asset = self.torch_client.get_asset(test_const.asset_id)
        latest_profile_status_asset = asset.get_latest_profile_status()
        assert latest_profile_status_asset is not None

    def test_add_tag(self):
        asset = self.torch_client.get_asset(test_const.asset_id)
        try:
            tag = asset.add_tag('test_tag')
            assert tag is not None, "Tag should not be None"
            print("Tag added successfully")
        except APIError as e:
            error_message = str(e)
            assert "A tag already exists with the same name" in error_message, \
                f"Unexpected error message: {error_message}"
            print(f"Expected error occurred: {error_message}")

    def test_cancel_profile(self):
        asset = self.torch_client.get_asset(test_const.asset_id)

        def operation():
            return asset.start_profile(ProfilingType.FULL)

        start_profile_asset = retry_operation(
            operation,
            test_const.MAX_RETRIES,
            test_const.RETRY_INTERVAL)
        time.sleep(15)
        profile_status = start_profile_asset.get_status()
        if profile_status['profileRequest']['status'] == 'IN PROGRESS':
            cancel_res = start_profile_asset.cancel()
            assert cancel_res is not None

    def test_execute_profile(self):
        asset = self.torch_client.get_asset(test_const.asset_id)

        def operation():
            return asset.start_profile(ProfilingType.FULL)

        start_profile_asset = retry_operation(
            operation,
            test_const.MAX_RETRIES,
            test_const.RETRY_INTERVAL)

        profile_status = start_profile_asset.get_status()
        assert profile_status is not None



from acceldata_sdk.models.span import Span, SpanEventType, SpanStatus
from acceldata_sdk.events.span_event import SpanEvent
from enum import Enum
import logging
from datetime import datetime

logger = logging.getLogger('span_context')
logger.setLevel(logging.INFO)


class SpanContextStatus(Enum):
    INITIALISED = 1
    STARTED = 2
    ABORTED = 3
    FINISHED = 4
    FAILED = 5


class SpanContext:

    def __init__(self, client=None, span: Span = None, parent=None, context_data=None, new_span: bool = None,
                 with_explicit_time: bool = False, *args, **kwargs):
        """
        Description:
            SpanContext is used for instantiate event for given span.
        :param client: TorchClient class instance
        :param span: Span class instance
        :param parent: SpanContext class instance which parent of current span context
        :param new_span: pass true value when its new span
        :param with_explicit_time: A boolean flag that determines how the span should be started.
        - If set to True, you can specify the start time for the span in subsequent span events.
        - If set to False, the span will be automatically started with the current time at the moment of creation.
        """
        self.client = client
        self.span = span
        self.parent = parent
        # self.status = SpanContextStatus.INITIALISED
        self.status = span.status
        self.started = True
        if new_span is None:
            new_span = False
        self.new_span = new_span
        # if self.status == SpanContextStatus.STARTED.name:
        #     self.started = True
        # else:
        logger.debug("SpanContext constructor invoked with the parameter with_explicit_time: ", with_explicit_time)
        if not with_explicit_time:
            if new_span or self.status == SpanContextStatus.INITIALISED.name:
                self.start(context_data=context_data)
        self.children = []

    def __repr__(self):
        return f"SpanContext({self.__dict__})"

    def is_span_finished(self):
        if self.span.status == SpanStatus.ABORTED.name or self.span.status == SpanStatus.FINISHED.name:
            logger.info('Can not send any event. Span is already completed')
            return True
        else:
            return False

    def start(self, context_data: dict = None, created_at: datetime = None):
        """
        Description:
            used to start span event
        :param context_data: context data of the span
        :param created_at: A datetime representing the explicit start time for the span.
         - If provided, the span will begin at the specified time.
         - If this parameter is not provided, the span will be started with the current time at the moment of creation.
        :return: SpanContextEvent class instance of the started span event
        """
        if self.is_span_finished():
            return
        if not self.started and self.status == SpanContextStatus.STARTED.name:
            logger.info('Can not start span that is already started')
            return
        span_event = SpanEvent()
        event_res = span_event.convert(
            event_uid=SpanEventType.SPAN_START.value,
            client=self.client,
            context_data=context_data,
            span_id=self.span.id,
            created_at = created_at
        )
        self.started = True
        self.started = SpanContextStatus.STARTED
        return event_res

    def end(self, context_data: dict = None, created_at: datetime = None):
        """
        Description:
            used to end span event
        :param context_data: context data of the span
         :param created_at: A datetime representing the explicit end time for the span.
         - If provided, the span will end at the specified time.
         - If this parameter is not provided, the span will be ended with the current time at the moment of creation.
        :return: SpanContextEvent class instance of the ended span event
        """
        if self.is_span_finished():
            return None
        if not self.started:
            logger.info('Can not end span that is not started.')
            return
        if self.status == SpanContextStatus.FINISHED:
            logger.info('Can not end span that is finished already.')
            return
        self.started = True
        span_event = SpanEvent()
        event_res = span_event.convert(
            event_uid=SpanEventType.SPAN_END.value,
            client=self.client,
            context_data=context_data,
            span_id=self.span.id,
            created_at=created_at
        )
        self.status = SpanContextStatus.FINISHED
        return event_res

    def abort(self, context_data: dict = None, created_at: datetime = None):
        """
        Description:
            used to abort span event
        :param context_data: context data of the span
        :param created_at: A datetime representing the explicit abort time for the span.
         - If provided, the span will abort at the specified time.
         - If this parameter is not provided, the span will be aborted with the current time at the moment of creation.
        :return: SpanContextEvent class instance of the aborted span event
        """
        if self.is_span_finished():
            return
        if not self.started:
            logger.info('Can not abort span that is not started yet')
            return
        if self.status == SpanContextStatus.ABORTED:
            logger.info('Can not abort span that is aborted already.')
            return
        span_event = SpanEvent()
        event_res = span_event.convert(
            event_uid=SpanEventType.SPAN_ABORTED.value,
            client=self.client,
            context_data=context_data,
            span_id=self.span.id,
            created_at = created_at
        )
        self.status = SpanContextStatus.ABORTED
        return event_res

    def failed(self, context_data: dict = None, created_at: datetime = None):
        """
        Description:
            used to fail span event
        :param context_data: context data of the span
        :param created_at: A datetime representing the explicit failed time for the span.
         - If provided, the span will fail at the specified time.
         - If this parameter is not provided, the span will be failed with the current time at the moment of creation.
        :return: SpanContextEvent class instance of the failed span event
        """
        if self.is_span_finished():
            return
        if not self.started:
            logger.info('Can not fail span that is not started yet')
            return
        if self.status == SpanContextStatus.FAILED:
            logger.info('Can not fail span that is aborted failed.')
            return
        span_event = SpanEvent()
        event_res = span_event.convert(
            event_uid=SpanEventType.SPAN_FAILED.value,
            client=self.client,
            context_data=context_data,
            span_id=self.span.id,
            created_at = created_at
        )
        self.status = SpanContextStatus.FAILED
        return event_res

    def send_event(self, span_event):
        if self.is_span_finished():
            return
        if not self.started:
            logger.info('Can not send custom event for given span context because span is not started yet. Kindly '
                        'start it '
                        'first.')
            return

        log_data = None
        if span_event.event_uid == 'LOG':
            log_data = span_event.log_data
        return span_event.convert(
            event_uid=span_event.event_uid,
            client=self.client,
            context_data=span_event.context_data,
            span_id=self.span.id,
            log_data=log_data,
            created_at=span_event.created_at
        )

    def create_child_span(self, uid: str, context_data: dict = None, associatedJobUids=None,
                          with_explicit_time: bool = False):
        """
            Used to create child span context of the current span context
        :param context_data: context to start span
        :param uid: uid of the new span you want to create
        :param associatedJobUids: list of string (job uids)
        :param with_explicit_time: A boolean flag that determines how the child span should be started.
        - If set to True,child span will be started at the specified time in the subsequent span events.
        - If set to False, the child span will be automatically started with the current time at the moment of creation.
        :return: spanContext class instance for the newly created span
        """
        if uid is None:
            Exception('To create a child span uid is required.')

        create_child_span = Span(
            uid=uid,
            pipelineRunId=self.span.pipelineRunId,
            parentSpanId=self.span.id
        )
        payload = create_child_span.__dict__
        payload['status'] = create_child_span.status.name
        if associatedJobUids is None:
            associatedJobUids = []
        elif not isinstance(associatedJobUids, list):
            raise Exception('associatedJobUids should be a list')
        child_span_payload = {'span': payload, 'associatedJobUids': associatedJobUids}
        res = self.client.create_span(self.span.pipelineRunId, child_span_payload)
        logger.debug("create_child_span method invoked with the parameter with_explicit_time: ", with_explicit_time)
        span_context = SpanContext(client=self.client, span=res, parent=self, context_data=context_data, new_span=True,
                                   with_explicit_time=with_explicit_time)
        self.children.append(span_context)
        return span_context

    def has_children(self):
        """
        Description:
            To check whether given spanContext has children or not.
        :return: True if it has children else False
        """
        return len(self.get_child_spans()) > 0

    def get_child_spans(self):
        """
        Description:
            To get child spans
        :return: list of child spans
        """
        return self.client.get_child_spans(self.span.id)

    def is_root(self):
        """
        Description:
            To check whether given spanContext is root or not.
        :return: True if it is root else False
        """
        return self.parent is None



from acceldata_sdk.torch_client import TorchClient
import logging
import pytest

from acceldata_sdk.models.common_types import PolicyExecutionRequest, ExecutionType, MarkerConfig, \
    AssetMarkerConfig, BoundsIdMarkerConfig, YunikornConfig, SparkResourceConfig, \
    RuleSparkSQLDynamicFilterVariableMapping, Mapping, BoundsDateTimeMarkerConfig, BoundsFileEventMarkerConfig, \
    TimestampBasedMarkerConfig
import test_constants as test_const
from test_commons import retry_operation
from acceldata_sdk.errors import APIError, TorchSdkException

# Setup logging configuration
logging.basicConfig(
    level=logging.INFO,  # Set the logging level to INFO
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    handlers=[
        logging.StreamHandler()  # Log to the console
    ]
)

LOGGER = logging.getLogger(__name__)

# Authenticate to Acceldata
torch_client = TorchClient(**test_const.torch_credentials)
LOGGER.info("Torch client connected")


@pytest.mark.order(1)
def test_execute_full_dq_request_backward_compatible():
    dq_policy = torch_client.get_policy(identifier=test_const.dq_policy_backward_compatible_name)
    LOGGER.info(f"DQ Policy ID: {dq_policy.id}")
    LOGGER.info(
        f" Executing FULL DQ using backward compatible approach:")

    def operation():
        return torch_client.execute_dq_rule(
            rule_id=dq_policy.id, incremental=True
        )

    result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    assert result is not None
    LOGGER.info(f"DQ Policy Execution Result: {result}")


@pytest.mark.order(2)
def test_execute_incremental_dq_request_backward_compatible():
    policy_id = test_const.incremental_dq_policy_name
    dq_policy = torch_client.get_policy(identifier=policy_id)
    LOGGER.info(
        f" Executing INCREMENTAL DQ using backward compatible approach:")

    def operation():
        return torch_client.execute_dq_rule(
            rule_id=dq_policy.id, incremental=True
        )

    policy_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    assert policy_execution_result is not None
    LOGGER.info(f"DQ Policy Execution Result: {policy_execution_result}")


@pytest.mark.order(3)
def test_execute_full_dq_using_policy_execution_request(request):
    policy_id = test_const.dq_policy_name
    policy = torch_client.get_policy(identifier=policy_id)
    request = PolicyExecutionRequest(executionType=ExecutionType.FULL)
    LOGGER.info(f"Executing FULL DQ using Policy execution request: {request}")

    def operation():
        return torch_client.execute_dq_rule(rule_id=policy.id, policy_execution_request=request)

    policy_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    assert policy_execution_result is not None
    LOGGER.info(f"DQ Policy Execution Result: {policy_execution_result}")


@pytest.mark.order(4)
def test_execute_incremental_dq_using_policy_execution_request(request):
    policy_id = test_const.dq_policy_name
    policy = torch_client.get_policy(identifier=policy_id)
    request = PolicyExecutionRequest(executionType=ExecutionType.INCREMENTAL)
    LOGGER.info(f"Executing INCREMENTAL DQ using Policy execution request: {request}")

    def operation():
        return torch_client.execute_dq_rule(rule_id=policy.id, policy_execution_request=request)

    policy_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )
    assert policy_execution_result is not None
    LOGGER.info(f"DQ Policy Execution Result: {policy_execution_result}")


@pytest.mark.order(5)
def test_execute_selective_dq_spark_sql_dynamic_filter_variable_mapping():
    dq_policy = torch_client.get_policy(identifier=test_const.spark_sql_policy_name)
    LOGGER.info("\n" + "=" * 20 + "execute_selective_dq: DQ Policy Details " + "=" * 20 + "\n")
    LOGGER.info(f"execute_selective_dq: DQ Policy : {dq_policy}")

    markerConfig = BoundsIdMarkerConfig(idColumnName="ID", fromId=0, toId=1000)
    assetMarkerConfig = AssetMarkerConfig(assetId=9667404, markerConfig=markerConfig)
    markerConfigs = [assetMarkerConfig]
    yunikornConfig = YunikornConfig(minExecutors=1, maxExecutors=2, executorCores=2, executorMemory="2g", driverCores=2,
                                    driverMemory="2g")
    # additionalConfiguration = {"spark.executor.memoryOverhead": "384"}
    sparkResourceConfig = SparkResourceConfig(yunikorn=yunikornConfig,
                                              additionalConfiguration={})
    ruleItemSelections = []
    mapping = Mapping(key="column_name", isColumnVariable=True, value="100")
    sparkSQLDynamicFilterVariableMapping = [RuleSparkSQLDynamicFilterVariableMapping(
        ruleName="SelectiveDQPolicysparkSQLDynamicFilterVariable",
        mapping=[mapping])]
    policy_execution_request = PolicyExecutionRequest(
        markerConfigs=markerConfigs,
        executionType=ExecutionType.SELECTIVE,
        sparkResourceConfig=sparkResourceConfig,
        sparkSQLDynamicFilterVariableMapping=sparkSQLDynamicFilterVariableMapping,
        ruleItemSelections=ruleItemSelections)
    LOGGER.info(
        f"Executing SELECTIVE DQ ID based with SPARK SQL DYNAMIC FILTER using Policy execution request: {policy_execution_request}")

    def operation():
        return torch_client.execute_dq_rule(rule_id=dq_policy.id,
                                            policy_execution_request=policy_execution_request)

    policy_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL
    )

    assert policy_execution_result is not None
    LOGGER.info(f"DQ Policy Execution Result: {policy_execution_result}")


@pytest.mark.order(6)
def test_execute_selective_dq_without_marker_config():
    dq_policy = torch_client.get_policy(identifier=test_const.spark_sql_policy_name)
    policy_execution_request = PolicyExecutionRequest(
        executionType=ExecutionType.SELECTIVE)
    LOGGER.info(
        f"Executing SELECTIVE DQ policy without marker config using Policy execution request: {policy_execution_request}")

    try:
        policy_execution_request = torch_client.execute_dq_rule(rule_id=dq_policy.id,
                                                                policy_execution_request=policy_execution_request)
    except TorchSdkException as e:
        error_message = str(e)
        assert "markerConfig is a mandatory parameter for execution type" in error_message, \
            f"Unexpected error message: {error_message}"
        LOGGER.warning(f"Expected error occurred: {error_message}")


@pytest.mark.order(7)
def test_execute_selective_dq_datetime_based():
    dq_policy = torch_client.get_policy(identifier=test_const.dq_policy_name)
    LOGGER.info("\n" + "=" * 20 + "execute_selective_dq: DQ Policy Details " + "=" * 20 + "\n")

    markerConfig = BoundsDateTimeMarkerConfig(dateColumnName="TO_DATE", format="yyyy-MM-dd",
                                              fromDate="2023-07-01 00:00:00.000", toDate="2024-07-14 23:59:59.999",
                                              timeZoneId="Asia/Calcutta")
    assetMarkerConfig = AssetMarkerConfig(assetId=9667404, markerConfig=markerConfig)
    markerConfigs = [assetMarkerConfig]
    policy_execution_request = PolicyExecutionRequest(
        markerConfigs=markerConfigs,
        executionType=ExecutionType.SELECTIVE)
    LOGGER.info(
        f"Executing SELECTIVE DQ DATETIME based using Policy execution request: {policy_execution_request}")

    def operation():
        return torch_client.execute_dq_rule(rule_id=dq_policy.id,
                                            policy_execution_request=policy_execution_request)

    policy_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL)
    assert policy_execution_result is not None
    LOGGER.info(policy_execution_result)


@pytest.mark.order(8)
def test_execute_selective_dq_file_event_based():
    dq_policy = torch_client.get_policy(identifier=test_const.file_event_based_dq)
    LOGGER.info("\n" + "=" * 20 + "execute_selective_dq: DQ Policy Details " + "=" * 20 + "\n")
    LOGGER.info(f"execute_selective_dq: DQ Policy ID: {dq_policy.id}")

    markerConfig = BoundsFileEventMarkerConfig(
        fromDate="2024-07-01 00:00:00.000", toDate="2024-07-01 23:59:59.999",
        timeZoneId="Asia/Calcutta")
    assetMarkerConfig = AssetMarkerConfig(assetId=1202688, markerConfig=markerConfig)
    markerConfigs = [assetMarkerConfig]
    policy_execution_request = PolicyExecutionRequest(
        markerConfigs=markerConfigs,
        executionType=ExecutionType.SELECTIVE)
    LOGGER.info(
        f"Executing SELECTIVE DQ FILE EVENT based using Policy execution request: {policy_execution_request}")

    def operation():
        return torch_client.execute_dq_rule(rule_id=dq_policy.id,
                                            policy_execution_request=policy_execution_request)

    policy_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL)
    assert policy_execution_result is not None
    LOGGER.info("\n" + "=" * 20 + "execute_selective_dq: DQ Policy Execution Result Details " + "=" * 20 + "\n")
    LOGGER.info(policy_execution_result)


@pytest.mark.order(9)
def test_execute_selective_dq_kafka_timestamp_based():
    dq_policy = torch_client.get_policy(identifier=test_const.kafka_dq_policy_name)

    markerConfig = TimestampBasedMarkerConfig(
        format="yyyy-mm-dd",
        initialOffset="2023-06-01",
        timeZoneId="Asia/Calcutta")
    assetMarkerConfig = AssetMarkerConfig(assetId=5241961, markerConfig=markerConfig)
    markerConfigs = [assetMarkerConfig]
    policy_execution_request = PolicyExecutionRequest(
        markerConfigs=markerConfigs,
        executionType=ExecutionType.SELECTIVE)
    LOGGER.info(
        f"Executing SELECTIVE DQ KAFKA TIMESTAMP based using Policy execution request: {policy_execution_request}")

    def operation():
        return torch_client.execute_dq_rule(rule_id=dq_policy.id,
                                                  policy_execution_request=policy_execution_request)

    policy_execution_result = retry_operation(
        operation,
        test_const.MAX_RETRIES,
        test_const.RETRY_INTERVAL)
    assert policy_execution_result is not None
    LOGGER.info("\n" + "=" * 20 + "execute_selective_dq: DQ Policy Execution Result Details " + "=" * 20 + "\n")
    LOGGER.info(policy_execution_result)



import re
from acceldata_sdk.constants import MIN_TORCH_BACKEND_VERSION_FOR_RULE_ID_API
from semantic_version import Version, SimpleSpec


class APIVersionUtils:

    def __init__(self):
        pass

    @staticmethod
    def _has_leading_zero(value: str) -> bool:
        """Checks if the version component has a leading zero."""
        return value and value[0] == '0' and value.isdigit() and value != '0'

    @staticmethod
    def extract_actual_version(torch_version) -> str:
        """Extracts the actual version from the given torch_version object."""
        version_string = torch_version.buildVersion
        if version_string:
            version_pattern = r'^(\d+)(?:\.(\d+)(?:\.(\d+))?)?(?:-(.*))?$'
            match = re.match(version_pattern, version_string)

            if not match:
                raise ValueError(f'Invalid version string: {version_string}')

            major, minor, patch, _ = match.groups()

            # Validate no leading zeroes in version parts
            for part_name, part in zip(['major', 'minor', 'patch'], [major, minor, patch]):
                if APIVersionUtils._has_leading_zero(part):
                    raise ValueError(f"Invalid leading zero in {part_name} part of version: {version_string}")

            # Convert to integers, treating missing parts as zeros
            major, minor, patch = int(major), int(minor or 0), int(patch or 0)
            actual_version = f'{major}.{minor}.{patch}'
            #TODO Revert after testing
            actual_version="3.12.0"
            return actual_version
        else:
            raise ValueError('Torch version is not available.')


    @staticmethod
    def validate_torch_version_for_rule_id_api(actual_version: str):
        """Validates if the given Torch version supports the rule ID API."""
        if Version(MIN_TORCH_BACKEND_VERSION_FOR_RULE_ID_API) not in SimpleSpec(f'<={actual_version}'):
            raise ValueError(f'The rule ID API is not supported for Torch version {actual_version}.')



import logging
import os

logging.basicConfig(level=logging.WARNING)

torch_client_credentials = {
    'url': os.getenv('TORCH_CATALOG_URL', 'https://torch.acceldata.local:5443'),
    'access_key': os.getenv('TORCH_ACCESS_KEY', 'OY2VVIN2N6LJ'),
    'secret_key': os.getenv('TORCH_SECRET_KEY', 'da6bDBimQfXSMsyyhlPVJJfk7Zc2gs')
}


from dataclasses import dataclass
from typing import List, Dict
from enum import Enum, auto

from acceldata_sdk.models.connection import Connection


@dataclass
class ConfigProperty:
    key = None
    value = None

    def __init__(self,
                 key=None,
                 value=None, *args, **kwargs):
        self.key = key
        self.value = value


@dataclass
class Crawler:
    name = None

    def __init__(self, name=None, *args, **kwargs):
        self.name = name

    def __repr__(self):
        return f"Crawler({self.__dict__})"


@dataclass
class DatasourceType:
    id = None
    name = None

    def __init__(self,
                 id=None,
                 name=None, *args, **kwargs):
        self.name = name
        self.id = id


@dataclass
class DatasourceSourceModel:
    id = None
    name = None

    def __init__(self,
                 id=None,
                 name=None, *args, **kwargs):
        self.name = name
        self.id = id


@dataclass
class DatasourceSourceType:

    def __init__(self, id, name, sourceModel=None, connectionTypeId=None, *args, **kwargs):
        """
            Description:
                Datasource source type
        :param id: id of the source type
        :param name: name of the source type
        :param sourceModel: source model
        :param connectionTypeId: (int) connection type id for the given source type
        """
        self.id = id
        self.name = name
        self.connectionTypeId = connectionTypeId
        if isinstance(sourceModel, dict):
            self.sourceModel = DatasourceSourceModel(**sourceModel)
        else:
            self.sourceModel = sourceModel

    def __repr__(self):
        return f"DatasourceSourceType({self.__dict__})"

class PropertyType(Enum):
    INT = auto()
    FLOAT = auto()
    STRING = auto()
    BOOLEAN = auto()
    JSON = auto()
    DURATION = auto()
    OBJECT = auto()
    OPTIONS = auto()
    URL = auto()
    URI = auto()
    PASSWORD = auto()
    EMAIL = auto()
    CRONEXPRESSION = auto()
    FILE = auto()
    ARRAY = auto()

    class SubPropertyType(Enum):
        STRING = auto()
        INT = auto()
        FLOAT = auto()
        BOOLEAN = auto()


class OptionTypes(Enum):

    CSV = auto()
    PARQUET = auto()
    ORC = auto()
    JSON = auto()
    AVRO = auto()
    CONFLUENT_AVRO = auto()

    PLAINTEXT = auto()
    SASL_PLAINTEXT = auto()
    SASL_SSL = auto()
    SSL = auto()
    BASIC_AUTH = auto()

    STANDALONE = auto()
    ORACLE_CLOUD_AUTONOMOUS_DB = auto()

    # Postgresql Environments
    ON_PREMISE = auto()
    AMAZON_RDS = auto()

    # Analysis Service Data Processor types
    HADOOP = auto()
    KUBERNETES = auto()
    DATABRICKS = auto()

    # Kafka Schema Naming Strategies
    TOPIC_NAME = auto()
    RECORD_NAME = auto()
    TOPIC_RECORD_NAME = auto()
    KEY = auto()
    VALUE = auto()

    # Torch Result Location Types
    HDFS = auto()
    S3 = auto()


@dataclass
class PropertyTemplate:
    key: str = None
    description: str = None
    displayLabel: str = None
    type: PropertyType = None
    options: List[OptionTypes] = None
    subType: PropertyType.SubPropertyType = None
    required: bool = None
    children: object = None
    visibility: str = None
    value: str = None
    replacementFor: str = None
    hidden: bool = None
    readOnly: bool = None
    configKey: str = None
    configValue: str = None
    multiSelect: bool = None


@dataclass
class PropertyTemplates:
    connections: Dict[str, List[PropertyTemplate]] = None
    assemblies: Dict[str, List[PropertyTemplate]] = None


@dataclass
class SourceModel:
    id = None,
    name = None

    def __init__(self,
                 id=None,
                 name=None, *args, **kwargs):
        self.name = name
        self.id = id


@dataclass
class SourceType:
    id: str = None
    name: str = None
    sourceModel: SourceModel = None
    connectionTypeId: str = None
    miniProfiling: bool = None
    propertyTemplates: PropertyTemplates = None


@dataclass
class SecurityConfig:
    type = None
    config = None

    def __init__(self,
                 type=None,
                 config=None, *args, **kwargs):
        self.type = type
        self.config = config


class DataSource:

    def __init__(self,
                 name: str = None,
                 isSecured: bool = None,
                 isVirtual: bool = None,
                 id: int = None,
                 createdAt: str = None,
                 updatedAt: str = None,
                 assemblyProperties = None,
                 conn: Connection = None,
                 connectionId: int = None,
                 crawler: Crawler = None,
                 currentSnapshot: str = None,
                 description: str = None,
                 sourceType: SourceType = None,
                 securityConfig: SecurityConfig = None,
                 schedule: str = None,
                 configuration=None,
                 client=None,
                 autoProfile: bool = None,
                 tenantId: str = None,
                 integrationId: str = None,
                 subIntegrationId: str = None,
                 isProtectedResource: bool = None,
                 createdBy: str = None,
                 *args, **kwargs
                 ):
        """
            Description:
                datasource class.
        :param name: name of the datasource
        :param isSecured: is secured or not
        :param isVirtual: is virtual datasource or not
        :param id: id of the datasource
        :param createdAt: creation time of the datasource
        :param updatedAt: updated time of the datasource
        :param assemblyProperties: datasource properties
        :param conn: connection details for the ds
        :param connectionId: connection id of the datasource
        :param crawler: crawler details of the datasource
        :param currentSnapshot: current version of the datasource
        :param description: desc of the datasource
        :param sourceType: (DatasourceSourceModel) source type details
        :param securityConfig: security configuration for the given ds
        :param schedule: scheduled exp
        :param configuration: configurations
        """
        self.name = name
        self.isSecured = isSecured
        self.isVirtual = isVirtual
        self.id = id
        self.createdAt = createdAt
        self.updatedAt = updatedAt
        self.assemblyProperties = assemblyProperties
        if isinstance(conn, dict):
            self.conn = Connection(**conn)
        else:
            self.conn = conn
        self.connectionId = connectionId
        if isinstance(crawler, dict):
            self.crawler = Crawler(**crawler)
        else:
            self.crawler = crawler
        self.currentSnapshot = currentSnapshot
        self.description = description
        if isinstance(securityConfig, dict):
            self.securityConfig = SecurityConfig(**securityConfig)
        else:
            self.securityConfig = securityConfig
        self.schedule = schedule
        self.configuration = configuration
        if isinstance(sourceType, dict):
            self.sourceType = SourceType(**sourceType)
        else:
            self.sourceType = sourceType
        self. autoProfile = autoProfile
        self.tenantId = tenantId
        self.integrationId = integrationId
        self.subIntegrationId = subIntegrationId
        self.isProtectedResource = isProtectedResource
        self.createdBy = createdBy

        self.client = client

    def __repr__(self):
        return f"DataSource({self.__dict__})"

    def start_crawler(self):
        return self.client.start_crawler(self.name)

    def get_crawler_status(self):
        return self.client.get_crawler_status(self.name)


class CrawlerStatus:
    def __init__(self, assemblyName, isSuccess=None, status=None, *args, **kwargs):
        self.assemblyName = assemblyName
        if status is not None:
            self.status = status
        if isSuccess is not None:
            self.isSuccess = isSuccess

    def __repr__(self):
        return f"CrawlerStatus({self.__dict__})"
